/******************************************************************************
*
* project name:    TI-Chess
* file name:       clocks.c
* initial date:    11/07/2000
* authors:         thomas.nussbaumer@gmx.net (coding)
*                  marcos.lopez@wol.es (design/graphics/beta testing)
* description:     contains data and handling routines for clocks
*
* number sprites converted using Image Studio v1.1 (Matt Johnson)
*
* $Id: clocks.c,v 1.2 2000/08/12 15:31:12 Thomas Nussbaumer Exp $
*
*******************************************************************************/


#if TI89_VERSION

//-------------------------------------
// definition of clock positions (TI89)
//-------------------------------------
#define CLOCK_START_X    116
#define CLOCK_START_Y0    91
#define CLOCK_START_Y1     2

#else

//-------------------------------------
// definition of clock positions (TI92)
//-------------------------------------
#define CLOCK_START_X    166
#define CLOCK_START_Y0    99
#define CLOCK_START_Y1    10

#endif



//------------------------------------------------------------------
//digital numbers for clocks (Both planes equal)
//------------------------------------------------------------------
unsigned char numbers[]={
0x30,0x48,0x48,0x00,0x48,0x48,0x30, //0
0x00,0x08,0x08,0x00,0x08,0x08,0x00, //1
0x30,0x08,0x08,0x30,0x40,0x40,0x30, //2
0x30,0x08,0x08,0x30,0x08,0x08,0x30, //3
0x00,0x48,0x48,0x30,0x08,0x08,0x00, //4
0x30,0x40,0x40,0x30,0x08,0x08,0x30, //5
0x30,0x40,0x40,0x30,0x48,0x48,0x30, //6
0x30,0x08,0x08,0x00,0x08,0x08,0x00, //7
0x30,0x48,0x48,0x30,0x48,0x48,0x30, //8
0x30,0x48,0x48,0x30,0x08,0x08,0x30};//9



//------------------------------------------------------------------
//inverted digital numbers for clocks used for inactive clock output
//------------------------------------------------------------------
unsigned char invnumbers[]={
~0x30,~0x48,~0x48,~0x00,~0x48,~0x48,~0x30, //0
~0x00,~0x08,~0x08,~0x00,~0x08,~0x08,~0x00, //1
~0x30,~0x08,~0x08,~0x30,~0x40,~0x40,~0x30, //2
~0x30,~0x08,~0x08,~0x30,~0x08,~0x08,~0x30, //3
~0x00,~0x48,~0x48,~0x30,~0x08,~0x08,~0x00, //4
~0x30,~0x40,~0x40,~0x30,~0x08,~0x08,~0x30, //5
~0x30,~0x40,~0x40,~0x30,~0x48,~0x48,~0x30, //6
~0x30,~0x08,~0x08,~0x00,~0x08,~0x08,~0x00, //7
~0x30,~0x48,~0x48,~0x30,~0x48,~0x48,~0x30, //8
~0x30,~0x48,~0x48,~0x30,~0x08,~0x08,~0x30};//9


//------------------------------------------------------------------
//misc. sprites for masking and inactive clock output
//------------------------------------------------------------------
unsigned char numclean[] = {0x03,0x03,0x03,0x03,0x03,0x03,0x03};
unsigned char numzeros[] = {0x0,0x0,0x0,0x0,0x0,0x0,0x0};
unsigned char numfill[]  = {0xfc,0xfc,0xfc,0xfc,0xfc,0xfc,0xfc};



//------------------------------------------------------------------
// structure describing a clock
//------------------------------------------------------------------
typedef struct {
    int x;              // x position of clock
    int y;              // y position of clock
    int digits[5];      // internally used
    int old_digits[5];  // internally used
} clock_t;


clock_t clocks[2]    = {};  // internal used clock structures for
                            // white and black
int     active_clock = 0;   // the active clock



/*===========================================================================*/
/* draws a 8x7 sprite in both planes                                         */
/*===========================================================================*/
void Sprite8x7Gray(int x,int y,unsigned char *sprite1,unsigned char *sprite2,int mode)
{
    long         addr1 = (long)plane1+30*y+((x>>3)&0xfffe),d1_1;
    long         addr2 = (long)plane2+30*y+((x>>3)&0xfffe),d2_1;
    unsigned int cnt  = 24-(x&15),data1,data2;
    int          h = 7;

    for (;h--;addr1+=30,addr2+=30) {
        data1=*sprite1++;
        data2=*sprite2++;
        if (mode==SPRT_AND) {
            *(long*)addr1&=~((long)(~data1&0xff)<<cnt);
            *(long*)addr2&=~((long)(~data2&0xff)<<cnt);
        }
        else {
            d1_1=(long)data1<<cnt;
            d2_1=(long)data2<<cnt;
            *(long*)addr1|=d1_1;
            *(long*)addr2|=d2_1;
        }
    }
}



/*===========================================================================*/
/* draws single digit                                                        */
/*===========================================================================*/
void DrawNumberOfClock(int nr,int clocknr) {
    int nr_offset[] = {0,7,12,19,24};
    int x;
    int y;
    int offset;

    if (clocks[clocknr].digits[nr] != 13) { //clocks[clocknr].old_digits[nr]) {
        x      = nr_offset[nr] + clocks[clocknr].x;
        y      = clocks[clocknr].y;
        offset = clocks[clocknr].digits[nr]*7;
        if (clocknr == active_clock) {
            Sprite8x7Gray(x,y,numclean,numclean,SPRT_AND);
            Sprite8x7Gray(x,y,&numbers[offset],&numbers[offset],SPRT_OR);
        }
        else {
            Sprite8x7Gray(x,y,numclean,numclean,SPRT_AND);
            Sprite8x7Gray(x,y,numfill,numzeros,SPRT_OR);
            Sprite8x7Gray(x,y,&invnumbers[offset],numclean,SPRT_AND);
            Sprite8x7Gray(x,y,numzeros,&numbers[offset],SPRT_OR);
        }
        clocks[clocknr].old_digits[nr] = clocks[clocknr].digits[nr];
    }
}



/*===========================================================================*/
/* draws a double-point of clock                                             */
/*===========================================================================*/
void DrawPointsOfClock(int clocknr) {
    int nr;

    for (nr=0;nr<2;nr++) {
        int x = clocks[clocknr].x+5;
        int y = clocks[clocknr].y;

        if (nr) x+=12;

        if (clocknr == active_clock) {
            Sprite8x7Gray(x,y,numclean,numclean,SPRT_AND);
        }
        else {
            Sprite8x7Gray(x,y,numclean,numclean,SPRT_AND);
            Sprite8x7Gray(x,y,numfill,numzeros,SPRT_OR);
        }
        x++;
        y+=2;
        SetPlane(0);
        if (clocknr == active_clock) {
            DrawPix(x,y,A_NORMAL);
            DrawPix(x,y+2,A_NORMAL);
        }
        else {
            DrawPix(x,y,A_REVERSE);
            DrawPix(x,y+2,A_REVERSE);
        }
        SetPlane(1);
        DrawPix(x,y,A_NORMAL);
        DrawPix(x,y+2,A_NORMAL);
    }
}



/*===========================================================================*/
/* update all numbers of a clock (redraw if changed)                         */
/*===========================================================================*/
void UpdateAllNumbers(int nr) {
    int i;
    for (i=0;i<5;i++) DrawNumberOfClock(i,nr);
}



/*===========================================================================*/
/* increments the active clock                                               */
/*===========================================================================*/
void IncrementClock() {
    int x;
    int y;
    int i;

    int *digits;

    x      = clocks[active_clock].x;
    y      = clocks[active_clock].y;
    digits = clocks[active_clock].digits;

    digits[4]++;
    for (i=4;i>=0;i--) {
        if (digits[i] == (10 - ((i%2)<<2))) {
            digits[i] = 0;
            if (i) digits[i-1]++;
        }
    }
    UpdateAllNumbers(active_clock);
}



/*===========================================================================*/
/* redraws both clocks                                                       */
/*===========================================================================*/
void RedrawClocks() {
    int tmp;
    int i;

    // hack to get a full refresh of all numbers of the clocks
    memset(clocks[0].old_digits,0xff,sizeof(int)*5);
    memset(clocks[1].old_digits,0xff,sizeof(int)*5);

    // redraw background of clocks
    for (tmp=0;tmp<2;tmp++) {
        int mode = A_REPLACE;
        int x    = clocks[tmp].x;
        int y    = clocks[tmp].y;

        if (tmp == active_clock) mode = A_REVERSE;

        SetPlane(0);
        DrawLine(x+1,y-1,x+28,y-1,mode);  // top
        DrawLine(x+1,y+7,x+28,y+7,mode);  // bottom
        DrawLine(x-1,y+1,x-1,y+5,mode);   // left
        DrawLine(x+30,y+1,x+30,y+5,mode); // right

        // fill the "which color" marking area
        for (i=0;i<2;i++) {
            SetPlane(i);
            mode = A_REPLACE;
            if (tmp==0) mode = A_REVERSE;
            DrawLine(x-12,y+1,x-3,y+1,mode);DrawLine(x+32,y+1,x+41,y+1,mode);
            DrawLine(x-13,y+2,x-3,y+2,mode);DrawLine(x+32,y+2,x+42,y+2,mode);
            DrawLine(x-13,y+3,x-3,y+3,mode);DrawLine(x+32,y+3,x+42,y+3,mode);
            DrawLine(x-13,y+4,x-3,y+4,mode);DrawLine(x+32,y+4,x+42,y+4,mode);
            DrawLine(x-12,y+5,x-3,y+5,mode);DrawLine(x+32,y+5,x+41,y+5,mode);
        }
    }

    // draw double points and numbers
    DrawPointsOfClock(0);
    DrawPointsOfClock(1);
    UpdateAllNumbers(0);
    UpdateAllNumbers(1);
}



/*===========================================================================*/
/* switches positions of clocks                                              */
/*===========================================================================*/
void SwitchClockPositions() {
    int y = clocks[0].y;

    clocks[0].y = clocks[1].y;
    clocks[1].y = y;
    RedrawClocks();
}



/*===========================================================================*/
/* Toggles the clocks                                                        */
/* if (reset == 1) the clocks will be reset to 0:00:00 and white clock       */
/* will be set active                                                        */
/*===========================================================================*/
void ToggleClocks(int reset,int inverted) {
    int tmp;
    if (reset) {
        memset(clocks[0].digits,0,sizeof(int)*5);
        memset(clocks[1].digits,0,sizeof(int)*5);
        clocks[0].x     = CLOCK_START_X;
        clocks[0].y     = CLOCK_START_Y0;
        clocks[1].x     = CLOCK_START_X;
        clocks[1].y     = CLOCK_START_Y1;
        active_clock    = 1;
    }

    if (active_clock) active_clock = 0;
    else              active_clock = 1;

    if (clocks[inverted].y != CLOCK_START_Y0) SwitchClockPositions();
    else                                      RedrawClocks();

}



//#############################################################################
//###################### NO MORE FAKES BEYOND THIS LINE #######################
//#############################################################################
//
//=============================================================================
// Revision History
//=============================================================================
//
// $Log: clocks.c,v $
// Revision 1.2  2000/08/12 15:31:12  Thomas Nussbaumer
// substitution keywords added
//
//
