/******************************************************************************
*
* project name:    TI-Chess
* file name:       gui.c
* initial date:    24/02/2000
* authors:         thomas.nussbaumer@gmx.net (coding)
*                  marcos.lopez@wol.es (design/graphics/beta testing)
* description:     contains most of the GUI stuff
*
* $Id: gui.c,v 1.2 2000/08/12 15:31:12 Thomas Nussbaumer Exp $
*
******************************************************************************/


//-----------------------------------------------------------------------------
// GLOBALS
//-----------------------------------------------------------------------------
SCR_RECT squarerect     = {};           // temporary rectangle
int      inverted       = FALSE;
int      select_x       =  0;           // selected square (x coordinate)
int      select_y       =  0;           // selected square (y coordinate)
int      marked_rect1   = -1;
int      marked_rect2   = -1;
int      idx_from       = -1;           // index into board of from-square
int      idx_to         = -1;           // index into board of to-square
int      select_promote = 0;            // used to store selected promotion fig
int      automatic_mode = FALSE;        // automatic mode setting
int      twoplayer_mode = FALSE;        // two player mode setting
char     jmp_buf[56]    = {};           // longjump for new and exit functions
char     do_exit        = 0;            // for exit() handling
char     drawn[BOARD_SIZE] = {};        // used to update board display
int      choice         = MENU_NONE;



//-----------------------------------------------------------------------------
// declarations of the necessary "external" routines and variables
//-----------------------------------------------------------------------------
extern char*         board;      // the board
extern int           act_color;  // which turn (white or black)
extern int           std_depth;  // standard depth
extern unsigned int  move_count; // number of moves so far
extern long          nr_nodes;
extern int           white_king_pos;
extern int           black_king_pos;
extern hash_t        act_hashkey;
extern int           already_mate;
extern store_t*      move_store;
extern int           moves_stored;
extern int           moves_forward;
extern defaults_t    defaults;
extern int           problem_active;
extern char          problem_text1[SIZE_TEXT+1];
extern char          problem_text2[SIZE_TEXT+1];
extern char          problem_text3[SIZE_TEXT+1];

int            AttacksField(int field,int side);
char*          Move2Str(move_t* pmove,int fromboard);
char*          Field2Notation(int fieldnumber,char* fieldnotation);
char           Figure2Char(char figure);
move_t*        ComputerMove(void);
int            UserMove(char* move,int updateboards);
void           InitEngine(int level);
void           TakeBackMove(int silent);
void           UndoTakeBackMove(int silent);
void           CalcHashKey(void);
void           InitHashCode(void);
int            HandleLoad(int automatic,int load_problem);
int            HandleSave(int automatic);
int            SaveDefaults();
int            MainMenu(int showmenu);



/*===========================================================================*/
/* restart the clocks and initialize infoboards                              */
/*===========================================================================*/
void ResetDisplays() {
    ToggleClocks(1,inverted);
    if (act_color==WHITE) {
        InitInfo(act_color,-1,std_depth,inverted);
    }
    else {
        InitInfo(act_color,std_depth,-1,inverted);
        ToggleClocks(0,inverted);
    }
}



/*===========================================================================*/
/* draws marking rounded rectangle arround given square (last computer move) */
/*===========================================================================*/
void DrawMarkingRect(int idx) {
    int      i;
    int      x;
    int      y;
    int      x0,x1,y0,y1;

    if (inverted) {
        y = idx / 10 - 2;
        x = 7 - (idx % 10 - 1);
    }
    else {
        y = 7 - (idx / 10 - 2);
        x = idx % 10 - 1;
    }

    x0 = OFFSET_X+SIZE_X*x;
    x1 = OFFSET_X+SIZE_X*(x+1)-1;
    y0 = OFFSET_Y+SIZE_Y*y;
    y1 = OFFSET_Y+SIZE_Y*(y+1)-1;

    DrawColorLine(x0+1,y0,  x1-1,y0,  COLOR_BLACK);
    DrawColorLine(x0+1,y1,  x1-1,y1,  COLOR_BLACK);
    DrawColorLine(x0,  y0+1,x0,  y1-1,COLOR_BLACK);
    DrawColorLine(x1,  y0+1,x1,  y1-1,COLOR_BLACK);
}



/*===========================================================================*/
/* maps cursor position to field value                                       */
/*===========================================================================*/
int Cursor2Idx(int x,int y) {
    if (inverted) return 21 + y*10+(7-x);
    else          return 21 + (7-y)*10+x;
}



/*===========================================================================*/
/* draws complete board                                                      */
/*===========================================================================*/
void DrawBoard(int hidecursor) {
    int i,j,idx;
    char val;

    // check all squares for redraw ...
    for (i=0;i<8;i++) {
        for (j=0;j<8;j++) {
            idx = Cursor2Idx(i,j);
            val = board[idx];
            if (drawn[idx] == val) {
                if (i!=select_x || j!=select_y) continue;
            }

            DrawSquare(i,j,val);

            drawn[idx] = val;

            if (idx == marked_rect1) {
               DrawMarkingRect(idx);
               drawn[idx]   = -OUTSIDE;
               marked_rect1 = -1;
            }
            else if (idx == marked_rect2) {
               DrawMarkingRect(idx);
               drawn[idx]   = -OUTSIDE;
               marked_rect2 = -1;
            }

            if (idx == idx_from) DrawMarkingRect(idx);

        }
    }

    //---------------------------------------------------
    // draw the cursor if wanted
    //---------------------------------------------------
    if (!hidecursor) {
        squarerect.xy.x0 = OFFSET_X+SIZE_X*select_x;
        squarerect.xy.x1 = OFFSET_X+SIZE_X*(select_x+1)-1;
        squarerect.xy.y0 = OFFSET_Y+SIZE_Y*select_y;
        squarerect.xy.y1 = OFFSET_Y+SIZE_Y*(select_y+1)-1;
        for (i=0;i<2;i++) {
            SetPlane(i);
            ScrRectFill(&squarerect,&fullscreen,A_XOR);
        }
    }
}



/*===========================================================================*/
/* initializes GUI                                                           */
/*===========================================================================*/
void InitGUI() {
    int i,j;

    select_x     =  0;
    select_y     =  7;
    idx_from     = -1;
    idx_to       = -1;
    marked_rect1 = -1;
    marked_rect2 = -1;

    FontSetSys(F_4x6);

    ClearInfoBoardData();
    OutputBackground();

    for (i=0;i<BOARD_SIZE;i++) drawn[i] = -OUTSIDE;

    InitHashCode();
}



/*===========================================================================*/
/* handles a request dialog (returns TRUE if 'y' is pressed)                 */
/*===========================================================================*/
int RequestDialog(char* request) {
     if (defaults.requests == OPTION_OFF) {
         return TRUE;
     }
     else {
         int read;
         int i;
         int j;
         int sy;
         int p;

         LCD_BUFFER LCD_tmp1;
         LCD_BUFFER LCD_tmp2;

         memcpy(LCD_tmp1,plane1,LCD_SIZE);
         memcpy(LCD_tmp2,plane2,LCD_SIZE);

         sy = LCD_HEIGHT/2 - 2;

         DrawPopup(4,sy-10,LCD_WIDTH-5,sy+10);
         DrawString(0,sy-3,request,F_6x8,A_REPLACE | A_CENTERED | A_SHADOWED);

         do {
             read = GetUserInput(0);
         }
         while (read != KEY_ESCAPE && read != KEY_ENTER && read != KEY_EXIT);

         memcpy(plane1,LCD_tmp1,LCD_SIZE);
         memcpy(plane2,LCD_tmp2,LCD_SIZE);

         if (read == KEY_ENTER) return TRUE;
         else                   return FALSE;
    }
}



/*===========================================================================*/
/* simple exit() implementation which does a long jump back into MainGUI()   */
/*===========================================================================*/
void LeavePrg(char* msg) {
    if (msg) RequestDialog(msg);
    do_exit = 1;
    longjmp(jmp_buf,1);
}



/*===========================================================================*/
/* corrects informations of the info board                                   */
/*===========================================================================*/
void CorrectInfoBoard() {
    if (twoplayer_mode) {
        InitInfo(act_color,-1,-1,inverted);
    }
    else {
        if (automatic_mode) {
            InitInfo(act_color,std_depth,std_depth,inverted);
        }
        else {
            if (act_color==WHITE) InitInfo(act_color,-1,std_depth,inverted);
            else                  InitInfo(act_color,std_depth,-1,inverted);
        }
    }
}



/*===========================================================================*/
/* handles a computer move                                                   */
/*===========================================================================*/
void HandleComputerMove() {
    move_t *executed;

    SetActiveInfo(act_color,std_depth,TRUE);
    executed = ComputerMove();
    UpdateNodeDisplay(nr_nodes);

    if (executed != 0) {
        marked_rect1 = executed->to;
        marked_rect2 = executed->from;
        if (!already_mate) {
            if (act_color == WHITE) {
                if (AttacksField(white_king_pos,BLACK)) OutputSpecial("CHECK !!!","");
            }
            else {
                if (AttacksField(black_king_pos,WHITE)) OutputSpecial("CHECK !!!","");
            }
        }

        if (automatic_mode) SetActiveInfo(act_color,std_depth,TRUE);
        else                SetActiveInfo(act_color,-1,TRUE);
        if (!already_mate)  ToggleClocks(0,inverted);
    }
    else {
        automatic_mode = FALSE;
        SetActiveInfo(act_color,-1,TRUE);
    }

}



/*===========================================================================*/
/* nomen omen est                                                            */
/*===========================================================================*/
void MajorInputLoop() {
    int  read;
    int  tmpfig;
    int  i;
    char tmpnotation[10];

    do {
        DrawBoard(automatic_mode);
        if (choice == MENU_NONE) {
            if (!automatic_mode) {
                read = GetUserInput(1);
                automatic_mode = FALSE;
            }
            else {
                // automatic mode goes here ...
                HandleComputerMove();
                continue;
            }
        }
        else {
            switch(choice) {
                case MENU_NEW:
                    longjmp(jmp_buf,1);
                    break;
                case MENU_PROBLEM:
                    HandleLoad(FALSE,TRUE);
                    ResetDisplays();
                    //if (HandleLoad(FALSE,TRUE)) {
                        //printf("Puzzle %03d loaded.\n- %s\n- %s\n",problem_active,
                        //        problem_text2,
                        //        problem_text3);
                    //}
                    //else {
                        //puts("Loading failed.\n");
                    //}
                    choice = MENU_NONE;
                    continue;
                case MENU_CMD_LOAD:
                    HandleLoad(FALSE,FALSE);
                    ResetDisplays();
                    //if (HandleLoad(FALSE,FALSE)) puts("Successfully loaded.\n");
                    //else                         puts("Loading failed.\n");
                    choice = MENU_NONE;
                    continue;
                case MENU_CMD_SAVE:
                    HandleSave(FALSE);
                    //if (HandleSave(FALSE)) puts("Successfully saved.\n");
                    //else                   puts("Saving failed.\n");
                    choice = MENU_NONE;
                    continue;
                case MENU_CMD_EXPORT:
                    HandleExport();
                    choice = MENU_NONE;
                    continue;
                case MENU_EXIT:
                    return;
                default:
                    choice = MENU_NONE;
                    continue;
            }
        }

        switch(read) {

            #if ENABLE_TESTKEY
            //--------------------------------------------------------
            // NOTE: this key ('=') should be only enabled for testing
            //--------------------------------------------------------
            case KEY_TESTING:
                //sprintf(s,"in %d mvs\n",(alpha + VALUE_MATE -1)/2);
                //OutputSpecial("","You'll die in 5 mvs");

                break;
            #endif

            case KEY_EXIT:
                LeavePrg(0);
                break;

            case KEY_ESCAPE:
                if (idx_from != -1) {
                    drawn[idx_from] = -OUTSIDE;
                    idx_from        = -1;
                }
                else {
                    choice = MainMenu(MENU_NONE);
                    for (i=0;i<BOARD_SIZE;i++) drawn[i] = -OUTSIDE;
                    DrawBoard(FALSE);
                    DrawInfoBoard(0);
                    DrawInfoBoard(1);
                }
                break;

            case KEY_MLEFT:
                drawn[Cursor2Idx(select_x,select_y)] = -OUTSIDE;
                select_x--;
                if (select_x < 0) select_x = 7;
                break;

            case KEY_MRIGHT:
                drawn[Cursor2Idx(select_x,select_y)] = -OUTSIDE;
                select_x++;
                if (select_x > 7) select_x = 0;
                break;

            case KEY_MDOWN:
                drawn[Cursor2Idx(select_x,select_y)] = -OUTSIDE;
                select_y++;
                if (select_y > 7) select_y = 0;
                break;

            case KEY_MUP:
                drawn[Cursor2Idx(select_x,select_y)] = -OUTSIDE;
                select_y--;
                if (select_y < 0) select_y = 7;
                break;

            case KEY_ENTER:
                if (already_mate) break;

                //----------------------------------------
                // we haven't marked a piece until now ...
                //----------------------------------------
                if (idx_from == -1) {
                    i      = Cursor2Idx(select_x,select_y);
                    tmpfig = board[i];
                    //------------------------------------
                    // check if we can mark this piece ...
                    //------------------------------------
                    if (tmpfig == EMPTY ||
                       (act_color==WHITE && tmpfig<0) ||
                       (act_color==BLACK && tmpfig>0)) break;

                    //--------------------------------------
                    // mark it and schedule it for redrawing
                    //--------------------------------------
                    idx_from        =   i;
                    drawn[idx_from] = -OUTSIDE;

                }
                else {
                    i = Cursor2Idx(select_x,select_y);
                    //---------------------------------------
                    // user want us to deselect the piece ...
                    //---------------------------------------
                    if (i == idx_from) {
                        idx_from = -1;
                        drawn[i] = -OUTSIDE;
                        break;
                    }

                    //---------------------------------------
                    // user want to finish its move
                    // check if move is allowed
                    //---------------------------------------
                    tmpfig = board[i];

                    if ((act_color==WHITE && tmpfig>0) ||
                        (act_color==BLACK && tmpfig<0)) break;

                    //----------------------------------
                    // move seems to be valid. Now check
                    // if its a pawn promotion move ...
                    //----------------------------------
                    idx_to = i;
                    tmpfig = board[idx_from];
                    if ((act_color==WHITE && tmpfig==W_PAWN && idx_from >= A7) ||
                        (act_color==BLACK && tmpfig==B_PAWN && idx_from <= H2))
                    {
                        select_promote = PromotionDialog();
                        // if exit key pressed -> leave processing immediately
                        if (KeyPressed() == KEY_EXIT) break;

                    }
                    else {
                       select_promote=0;
                    }

                    //-------------------------------------------------
                    // generate notation for move and try to execute it
                    //-------------------------------------------------
                    Field2Notation(idx_from,tmpnotation);
                    Field2Notation(idx_to,&tmpnotation[2]);
                    tmpnotation[4]  = select_promote;
                    tmpnotation[5] = 0;

                    if (!UserMove(tmpnotation,TRUE)) {
                        RequestDialog("!! Invalid Move !!");
                    }
                    else {
                        drawn[idx_from] = -OUTSIDE;
                        idx_from        = -1;
                        ToggleClocks(0,inverted);
                        if (!twoplayer_mode) {
                            DrawBoard(TRUE);
                            HandleComputerMove();
                        }
                        else {
                            SetActiveInfo(act_color,-1,TRUE);
                        }
                    }
                }
                break;

            case KEY_HELP:
                MainMenu(MENU_HELP);
                break;

            case KEY_SHOWMOVES:
                MainMenu(MENU_SHOWMOVES);
                break;

            case KEY_AUTOMATIC:
                if (!automatic_mode) {
                   if (RequestDialog("Start Automatic (Y/N)?")) {
                      automatic_mode = TRUE;
                      twoplayer_mode = FALSE;
                      if (idx_from != -1) {
                          drawn[idx_from] = -OUTSIDE;
                          idx_from        = -1;
                          DrawBoard(FALSE);
                      }
                   }
                }
                else {
                   automatic_mode = FALSE;
                }
                CorrectInfoBoard();
                break;

            case KEY_LOADSAVE:
                choice = MainMenu(MENU_LOADSAVE);
                break;

            case KEY_BACK:
                if (idx_from != -1) {
                    i = Cursor2Idx(select_x,select_y);
                    //---------------------------------------
                    // user want us to deselect the piece ...
                    //---------------------------------------
                    idx_from  = -1;
                    drawn[i]  = -OUTSIDE;
                }

                ClearInfoBoardData();
                TakeBackMove(FALSE);
                already_mate = FALSE;
                break;

            case KEY_FORWARD:
                if (idx_from != -1) {
                    i = Cursor2Idx(select_x,select_y);
                    //---------------------------------------
                    // user want us to deselect the piece ...
                    //---------------------------------------
                    idx_from  = -1;
                    drawn[i]  = -OUTSIDE;
                }
                UndoTakeBackMove(FALSE);
                break;

            case KEY_CHANGESIDE:
                if (RequestDialog("Change Sides (Y/N)?")) {
                    // to get a refresh of the info board
                    if (act_color == WHITE) SetActiveInfo(BLACK,-1,TRUE);
                    else                    SetActiveInfo(WHITE,-1,TRUE);
                    DrawBoard(TRUE);
                    HandleComputerMove();
                }
                break;


            case KEY_LEVELUP:
                std_depth++;
                if (std_depth > MAX_USERLEVEL) std_depth = MAX_USERLEVEL;
                defaults.strength = std_depth-1;
                SaveDefaults();
                CorrectInfoBoard();
                break;


            case KEY_LEVELDOWN:
                std_depth--;
                if (std_depth < MIN_USERLEVEL) std_depth = MIN_USERLEVEL;
                defaults.strength = std_depth-1;
                SaveDefaults();
                CorrectInfoBoard();
                break;


            case KEY_TWOPLAYER:
                if (twoplayer_mode) {
                    if (RequestDialog("ONE Player (Y/N)?")) {
                        twoplayer_mode = FALSE;
                    }
                }
                else {
                    if (RequestDialog("TWO player (Y/N)?")) {
                        twoplayer_mode = TRUE;
                    }
                }
                CorrectInfoBoard();
                break;

            case KEY_ROTATE:
                if (inverted) inverted = FALSE;
                else          inverted = TRUE;

                for (i=0;i<BOARD_SIZE;i++) drawn[i] = -OUTSIDE;
#if TI92_VERSION
                // on the TI92 we can draw coordinates
                DrawCoordinates(inverted);
#endif
                DrawBoard(FALSE);
                SwitchClockPositions();
                SwitchInfoBoards();
                break;
        }
    }
    while (1);
}



/*===========================================================================*/
/* main routine of the GUI                                                   */
/*===========================================================================*/
void MainGUI(void) {
    int p;
    do_exit = 0;
    choice  = MENU_NONE;

    //---------------------------------------------------
    // here we could add program-first-time-used-features
    //---------------------------------------------------
    if (defaults.firsttime == OPTION_ON) {
        defaults.firsttime = OPTION_OFF;
        SaveDefaults();
    }

    setjmp(jmp_buf);

    if (do_exit == 0) {
        InitGUI();
        automatic_mode = FALSE;
        twoplayer_mode = FALSE;
        InitEngine(defaults.strength+1);

        if (choice != MENU_NEW && defaults.autoload == OPTION_ON) HandleLoad(TRUE,FALSE);

        if (choice == MENU_NEW) choice = MENU_NONE;

#if TI92_VERSION
        // on the TI92 we can draw coordinates
        DrawCoordinates(inverted);
#endif

        ResetDisplays();
        MajorInputLoop(choice);
    }

    if (defaults.autosave == OPTION_ON) HandleSave(TRUE);
}



//#############################################################################
//###################### NO MORE FAKES BEYOND THIS LINE #######################
//#############################################################################
//
//=============================================================================
// Revision History
//=============================================================================
//
// $Log: gui.c,v $
// Revision 1.2  2000/08/12 15:31:12  Thomas Nussbaumer
// substitution keywords added
//
//
