/******************************************************************************
*
* project name:    TI-Chess
* file name:       tichess.c
* initial date:    20/02/2000
* authors:         thomas.nussbaumer@gmx.net (coding)
*                  marcos.lopez@wol.es (design/graphics/beta testing)
* description:     the entry point and the special clearscreen routine
*
* $Id: tichess.c,v 1.2 2000/08/12 15:31:13 Thomas Nussbaumer Exp $
*
******************************************************************************/

//#############################################################################
// +++ IMPORTANT +++ IMPORTANT +++ IMPORTANT +++ IMPORTANT +++ IMPORTANT +++
// +++ IMPORTANT +++ IMPORTANT +++ IMPORTANT +++ IMPORTANT +++ IMPORTANT +++
// +++ IMPORTANT +++ IMPORTANT +++ IMPORTANT +++ IMPORTANT +++ IMPORTANT +++
//
// Due to heavy problems in the past TI-Chess will not longer support an own
// DOORS version. The NOSTUB version will work under each kernel and OS.
//
// And note: don't try to start TI-Chess from any explorer-like program,
//           because almost all explorer-like programs cannot handle large
//           programs correctly !!
//
// +++ IMPORTANT +++ IMPORTANT +++ IMPORTANT +++ IMPORTANT +++ IMPORTANT +++
// +++ IMPORTANT +++ IMPORTANT +++ IMPORTANT +++ IMPORTANT +++ IMPORTANT +++
// +++ IMPORTANT +++ IMPORTANT +++ IMPORTANT +++ IMPORTANT +++ IMPORTANT +++
//#############################################################################


//-----------------------------------------------------------------------------
// this define is for batch compiling where the defines are specified
// at the commandline
//-----------------------------------------------------------------------------
#ifndef AUTO_BUILD

//=============================================================================
// THE FOLLOWING PRECOMPILER DEFINES ARE USED TO SET A SPECIFIC CALCULATOR TYPE
//
// !!!!!! NOTE: ONLY ONE OF THE DEFINES MAY BE SET TO '1' !!!!!
//=============================================================================
#define TI89_VERSION      0    // generate version for TI89
#define TI92_VERSION      1    // generate version for TI92

#endif



#define SAVE_SCREEN      // restore screen after program end

#if TI92_VERSION
#define USE_GRAY_PATCH   // use grayscale support from TIGCC
#endif

#include <nostub.h>

#if TI89_VERSION
#include "gray_ti89.h"   // use own (better) grayscale support
#endif


#include <all.h>

#include "version.h"
#include "tichess.h"
#include "logout.h"      // contains macros for log writing to link port
#include "interrupt.h"   // own key handler stuff
#include "input.h"       // key definitions
#include "generic.h"     // generic graphics function definitions


//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
// due to a very strange behavior of TIGCC we cannot use separate object until
// yet (the globals are not initialized correctly - NOT even the constants !!)
//
// So we've to go that really silly "include all into one" way ...
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

unsigned int *plane1      = 0;
unsigned int *plane2      = 0;
SCR_RECT fullscreen       = {0, 0, 239, 127}; // used for fullscreen clipping

#include "waitms.c"     // WaitForMillis() utility function
#include "interrupt.c"  // own key handler stuff
#include "generic.c"    // generic drawing functions
#include "board.c"      // pieceset data and output routine for single square
#include "infoboards.c" // handling routines for infoboards
#include "clocks.c"     // clocks data and routines
#include "input.c"      // user input handling
#include "logo.c"       // splashscreen data and output routine

#if TI89_VERSION
#include "bg_ti89.c"    // background data and output routine
#else
#include "bg_ti92.c"    // background data and output routine
#endif

#include "loadsave.c"   // loading/saving stuff
#include "gui.c"        // gui stuff
#include "engine.c"     // engine stuff
#include "hash.c"       // hashtable stuff
#include "menu.c"       // menu stuff

#if ENABLE_LOGGING
#include "logout.c"   // includes logging routines if necessary
#endif


#if TI92_VERSION
int _ti92plus;        // needed to produce ti92plus code
#else
int _ti89;            // needed to produce ti89 code
#endif

//--------------------------------------------------
// declarations of the necessary "external" routines
//--------------------------------------------------
extern char default_hashmode;

int  InitMemory();
void FreeMemory();
void MainGUI();
int  LoadDefaults();



/*===========================================================================*/
/* special effect clear screen                                               */
/*===========================================================================*/
void FXClearScreen() {
    int height  = LCD_HEIGHT;
    int height2 = height/2;
    int width   = LCD_WIDTH;
    int width2  = width/2;
    int i,j;

    //---------------------
    // clear in y direction
    //---------------------
    for (i=0;i<=height2;i++) {
        DrawLine(0,i,width,i,A_REPLACE);
        DrawLine(0,height-i,width,height-i,A_REPLACE);
        DrawLine(0,i-1,width,i-1,A_REVERSE);
        DrawLine(0,height-i+1,width,height-i+1,A_REVERSE);
        for (j=0;j<=10000;j++);
    }

    //-----------------------------
    // clear in x direction
    //-----------------------------
    for (i=0;i<=width2;i++) {
        DrawPix(i,height2,A_REVERSE);
        DrawPix(width-i,height2,A_REVERSE);
        for (j=0;j<=3000;j++);
    }
}



/*===========================================================================*/
/* due to a real strange behaviour the contrast gets lost when we set our    */
/* keyboard handler !!!                                                      */
/* I haven't managed to restore it correctly in the interrupt installer so I */
/* will do it just in the main routine ...                                   */
/*                                                                           */
/* The following routine fetches the address where the contrast will be      */
/* stored, so we can get and restore it later again                          */
/*===========================================================================*/
unsigned char* ActiveContrastAddr(void);

asm("ActiveContrastAddr: movem.l %d0-%d1,-(%a7)
         move.w          #4,%d0
         trap            #9
         movem.l (%a7)+,%d0-%d1
         rts");



/*===========================================================================*/
/* MAIN - where all the fun starts ...                                       */
/*===========================================================================*/
void _main(void) {
    int           i;
    char*         s1 = "ERROR: Memory";
    char*         s2 = "Insufficient Memory.";
    char          s3[30];
    unsigned int  size;
    unsigned char initial_contrast;

    initial_contrast = *ActiveContrastAddr(); // get the initial contrast

    LoadDefaults();   // load defaults from ticcfg

    default_hashmode = defaults.hashtables;

    FXClearScreen();

#if SHOW_MEM_USAGE
    if (default_hashmode == OPTION_ON) size = sizeof(hentry_t)*HASHSIZE*2;
    else                               size = 0;

    size += sizeof(char)*BOARD_SIZE + \
            sizeof(int)*MAX_DEPTH+sizeof(move_t)*(STACK_SIZE+1) + \
            sizeof(int)*MAX_DEPTH+sizeof(killer_t)*MAX_DEPTH + \
            sizeof(bicolor_t)*BOARD_SIZE + \
            sizeof(bicolor_t)*(H_COLUMN+2)*2 + \
            sizeof(int)*MAX_DEPTH*4 + \
            sizeof(fromto_t*)*MAX_DEPTH + \
            sizeof(fromto_t)*MAX_DEPTH*MAX_DEPTH + \
            sizeof(hash_t*)*12 + \
            sizeof(hash_t)*64*12 + \
            sizeof(store_t)*MAX_MOVES_STORED;
#endif
    FontSetSys(F_6x8);

    if (!InitMemory()) {
        FreeMemory();

        size = sizeof(char)*BOARD_SIZE + \
           sizeof(int)*MAX_DEPTH+sizeof(move_t)*(STACK_SIZE+1) + \
           sizeof(int)*MAX_DEPTH+sizeof(killer_t)*MAX_DEPTH + \
           sizeof(bicolor_t)*BOARD_SIZE + \
           sizeof(bicolor_t)*(H_COLUMN+2)*2 + \
           sizeof(int)*MAX_DEPTH*4 + \
           sizeof(fromto_t*)*MAX_DEPTH + \
           sizeof(fromto_t)*MAX_DEPTH*MAX_DEPTH + \
           sizeof(hash_t*)*12 + \
           sizeof(hash_t)*64*12 + \
           sizeof(store_t)*MAX_MOVES_STORED;

        sprintf(s3,"(Req'd %u Bytes)",size);

        FontSetSys(F_8x10);
        DrawStr((ScrRect->xy.x1-DrawStrWidth(s1,F_8x10))/2,ScrRect->xy.y1/3,s1,A_REPLACE);
        FontSetSys(F_6x8);
        DrawStr((ScrRect->xy.x1-DrawStrWidth(s2,F_6x8))/2,ScrRect->xy.y1/2,s2,A_REPLACE);
        DrawStr((ScrRect->xy.x1-DrawStrWidth(s3,F_6x8))/2,ScrRect->xy.y1*2/3,s3,A_REPLACE);
        ngetchx();
        return;
    }

#if SHOW_MEM_USAGE
    sprintf(s3,"[using %u Bytes]",size);
    FontSetSys(F_6x8);
    DrawStr(0,0,s3,A_REPLACE);
    DrawStr(0,10,"press a key",A_REPLACE);
    ngetchx();
#endif

    //----------------------------------------------------------
    // install own key handler (MUST BE DONE BEFORE GRAYMODE ON)
    //----------------------------------------------------------
    InstallInterruptHandlers();

    //---------------------------
    // turn on grayscale graphics
    //---------------------------
    if (!GrayMode(GRAY_ON)) {
        SaveDefaults();
        RestoreInterruptHandlers();
        return;
    }

    //---------------------------------
    // !!!!! restore the contrast !!!!!
    //---------------------------------
    *ActiveContrastAddr() = initial_contrast;

    //-----------------------------------------
    // setup bitplane globals and show the logo
    //-----------------------------------------
    plane1 = GetPlane(0);
    plane2 = GetPlane(1);

    Logo();

    //-----------------------------------------------
    // start timer for clocks and enter the main loop
    //-----------------------------------------------
    OSFreeTimer(USER_TIMER);
    OSRegisterTimer(USER_TIMER,20);

    MainGUI();

    //-----------------------------------------------
    // stop timer again, turn grayscales off, restore
    // old interrupt handlers again and free memory
    //-----------------------------------------------
    OSFreeTimer(USER_TIMER);
    GrayMode(GRAY_OFF);
    RestoreInterruptHandlers();
    FreeMemory();

    //------------------------------------------
    // give the system keyboard handler time to
    // recover and then read out all keypresses
    //------------------------------------------
    WaitForMillis(200);
    while (kbhit()) ngetchx(); // remove any pending keys
}



//#############################################################################
//###################### NO MORE FAKES BEYOND THIS LINE #######################
//#############################################################################
//
//=============================================================================
// Revision History
//=============================================================================
//
// $Log: tichess.c,v $
// Revision 1.2  2000/08/12 15:31:13  Thomas Nussbaumer
// substitution keywords added
//
//
