/******************************************************************************
*
* project name:    TI-Chess
* file name:       ticstart.c
* initial date:    23/03/2000
* author:          Zeljko Juric (zjuric@utic.net.ba)
*                  thomas.nussbaumer@gmx.net (misc. fixes)
* description:     launcher for tichess to overcome 8kB limit on AMS 2.03 and
*                  the 24kB limit on AMS 2.04/2.05
*
* THANX TO ZELJKO JURIC WHO RELEASES THIS CODE AS PART OF HIS SCOTTFREE DRIVER
* AND GAVE ME PERMISSION TO RE-USE IT
*
*
* NOTE: Since AMS 2.04 there exists a problem on HW2 calculators which
*       may crash the calculator if the main program is not archived.
*       This behavior shows up with all large programs and forces me
*       to integrate the check if the program is archived.
*       If the program is not archived ticstart() will not start tichess
*       if it detects an active AMS 2.04 or greater
*       
*       
*
* $Id: ticstart.c,v 1.3 2000/08/17 22:07:48 Thomas Nussbaumer Exp $
*
******************************************************************************/

#include <nostub.h>
#include <alloc.h>
#include <mem.h>
#include <vat.h>
#include <statline.h>
#include <system.h>
#include <error.h>
#include <compat.h>

#ifndef AUTO_BUILD
int _ti89, _ti92plus;         // Produce both .89z and .9xz files
#else
#if !TI89_VERSION
#if !TI92_VERSION
#error: Neither "TI89_VERSION" nor "TI92_VERSION" defined
#endif
#endif
#if TI92_VERSION
int _ti92plus;
#endif
#if TI89_VERSION
int _ti89;
#endif
#endif

#define fatal(s) {ST_showHelp(s); return;}


//--------------------------------------------------------------------------
// Note: This program is dirty-written to be as small as possible, so trying
// to understand how it works may be a good brain training for C experts :-)
//--------------------------------------------------------------------------
void _main(void){  
    char      *fptr,*cptr;  
    unsigned  *pptr,*eptr,plen,offs;
    SYM_ENTRY *SymPtr = DerefSym(SymFind($(tichess)));

    if (!SymPtr) fatal("tichess not found");

    plen = *(int*)(cptr=fptr=HLock(SymPtr->handle))+3;
  
    if (AMS_2xx && !SymPtr->flags.bits.archived) {
        HeapUnlock(SymPtr->handle);
        fatal("Please archive tichess");
    }
  
    if (SymPtr->flags.bits.archived || SymPtr->flags.bits.locked) {
        if (!(cptr=malloc(plen))) {
            HeapUnlock(SymPtr->handle);
            fatal("Out of memory");
        }    
        memcpy(cptr,fptr,plen);    
    }

    // relocate "by hand" for sure ...
    for (eptr=pptr=(unsigned*)(cptr+plen-4);*pptr;pptr-=2);
    while (pptr<eptr) offs=*++pptr,*(char**)(cptr+2+*++pptr)=cptr+0x40002+offs;

    ASM_call(cptr+0x40002);
    if (cptr!=fptr) free(cptr);

    HeapUnlock(SymPtr->handle);
}



//#############################################################################
//###################### NO MORE FAKES BEYOND THIS LINE #######################
//#############################################################################
//
//=============================================================================
// Revision History
//=============================================================================
//
// $Log: ticstart.c,v $
// Revision 1.3  2000/08/17 22:07:48  Thomas Nussbaumer
// (1)workaround for AMS 2.0x added to prevent calcs from crashing
//    (won't start if tichess is not archived)
// (2) missing HeapUnlock()'s added
//
// Revision 1.2  2000/08/12 15:31:13  Thomas Nussbaumer
// substitution keywords added
//
//
